﻿# -*- coding: utf-8 -*-
#######################################################################################################
#
#    MediaPortal for Dreambox OS
#
#    Coded by MediaPortal Team (c) 2013-2020
#
#  This plugin is open source but it is NOT free software.
#
#  This plugin may only be distributed to and executed on hardware which
#  is licensed by Dream Property GmbH. This includes commercial distribution.
#  In other words:
#  It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
#  to hardware which is NOT licensed by Dream Property GmbH.
#  It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
#  on hardware which is NOT licensed by Dream Property GmbH.
#
#  This applies to the source code as a whole as well as to parts of it, unless explicitely
#  stated otherwise.
#
#  If you want to use or modify the code or parts of it, permission from the authors is necessary.
#  You have to keep OUR license and inform us about any modification, but it may NOT be distributed
#  other than under the conditions noted above.
#
#  As an exception regarding modifcations, you are NOT permitted to remove
#  any copy protections implemented in this plugin or change them for means of disabling
#  or working around the copy protections, unless the change has been explicitly permitted
#  by the original authors. Also decompiling and modification of the closed source
#  parts is NOT permitted.
#
#  Advertising with this plugin is NOT allowed.
#
#  For other uses, permission from the authors is necessary.
#
#######################################################################################################

from Plugins.Extensions.MediaPortal.plugin import _
from Plugins.Extensions.MediaPortal.resources.imports import *
from Plugins.Extensions.MediaPortal.resources.decrypt import decrypturl

agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36'
cookies = CookieJar()
default_cover = "file://%s/camhub.png" % (config_mp.mediaportal.iconcachepath.value + "logos")

class camhubGenreScreen(MPScreen):

	def __init__(self, session):
		MPScreen.__init__(self, session, skin='MP_Plugin', default_cover=default_cover)

		self["actions"] = ActionMap(["MP_Actions"], {
			"ok" : self.keyOK,
			"0" : self.closeAll,
			"cancel" : self.keyCancel
		}, -1)

		self['title'] = Label("Camhub.cc")
		self['ContentTitle'] = Label("Genre:")
		self.keyLocked = True

		self.suchString = ''

		self.filmliste = []
		self.ml = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self['liste'] = self.ml

		self.onLayoutFinish.append(self.loadPage)

	def loadPage(self):
		self.filmliste = []
		self.keyLocked = True
		url = "http://www.camhub.cc/categories/"
		twAgentGetPage(url, agent=agent).addCallback(self.genreData).addErrback(self.dataError)

	def genreData(self, data):
		Cats = re.findall('class="item" href="(.*?)" title="(.*?)"', data, re.S)
		if Cats:
			for (Url, Title) in Cats:
				self.filmliste.append((upperString(Title), Url))
			self.filmliste.sort()
		self.filmliste.insert(0, ("Top Rated", "http://www.camhub.cc/top-rated/"))
		self.filmliste.insert(0, ("Most Viewed", "http://www.camhub.cc/most-popular/"))
		self.filmliste.insert(0, ("Newest", "http://www.camhub.cc/latest-updates/"))
		self.filmliste.insert(0, ("--- Search ---", "callSuchen"))
		self.ml.setList(map(self._defaultlistcenter, self.filmliste))
		self.keyLocked = False

	def keyOK(self):
		if self.keyLocked:
			return
		Name = self['liste'].getCurrent()[0][0]
		Link = self['liste'].getCurrent()[0][1]
		if Name == "--- Search ---":
			self.suchen()
		elif Link:
			self.session.open(camhubFilmScreen, Link, Name)

	def SuchenCallback(self, callback = None):
		if callback is not None and len(callback):
			Name = "--- Search ---"
			self.suchString = callback
			Link = urllib.quote(self.suchString).replace(' ', '-')
			self.session.open(camhubFilmScreen, Link, Name)

class camhubFilmScreen(MPScreen, ThumbsHelper):

	def __init__(self, session, Link, Name):
		self.Link = Link
		self.Name = Name
		MPScreen.__init__(self, session, skin='MP_Plugin', default_cover=default_cover)
		ThumbsHelper.__init__(self)

		self["actions"] = ActionMap(["MP_Actions"], {
			"ok" : self.keyOK,
			"0" : self.closeAll,
			"cancel" : self.keyCancel,
			"5" : self.keyShowThumb,
			"up" : self.keyUp,
			"down" : self.keyDown,
			"right" : self.keyRight,
			"left" : self.keyLeft,
			"nextBouquet" : self.keyPageUp,
			"prevBouquet" : self.keyPageDown,
			"green" : self.keyPageNumber
		}, -1)

		self['title'] = Label("Camhub.cc")
		self['ContentTitle'] = Label("Genre: %s" % self.Name)
		self['F2'] = Label(_("Page"))

		self['Page'] = Label(_("Page:"))
		self.keyLocked = True
		self.page = 1
		self.lastpage = 1

		self.filmliste = []
		self.ml = MenuList([], enableWrapAround=True, content=eListboxPythonMultiContent)
		self['liste'] = self.ml

		self.onLayoutFinish.append(self.loadPage)

	def loadPage(self):
		self.keyLocked = True
		self['name'].setText(_('Please wait...'))
		self.filmliste = []
		if re.match(".*Search", self.Name):
			url = "http://www.camhub.cc/search/%s/?mode=async&function=get_block&block_id=list_videos_videos_list_search_result&q=%s&category_ids=&sort_by=post_date&from_videos=%s&from_albums=%s" % (self.Link, self.Link.replace('-','+'), str(self.page), str(self.page))
		else:
			if "categories" in self.Link:
				url = "%s?mode=async&function=get_block&block_id=list_videos_common_videos_list&sort_by=post_date&from=%s" % (self.Link, str(self.page))
			elif "latest-updates" in self.Link:
				url = "%s?mode=async&function=get_block&block_id=list_videos_latest_videos_list&sort_by=post_date&from=%s" % (self.Link, str(self.page))
			elif "top-rated" in self.Link:
				url = "%s?mode=async&function=get_block&block_id=list_videos_common_videos_list&sort_by=rating&from=%s" % (self.Link, str(self.page))
			elif "most-popular" in self.Link:
				url = "%s?mode=async&function=get_block&block_id=list_videos_common_videos_list&sort_by=video_viewed&from=%s" % (self.Link, str(self.page))
		twAgentGetPage(url, agent=agent, cookieJar=cookies).addCallback(self.genreData).addErrback(self.dataError)

	def genreData(self, data):
		self.getLastPage(data, 'lass="pagination"(.*?)>Last<', '.*[from:|>](\d+)["|<]')
		Movies = re.findall('class="item\s{0,3}".*?href="(.*?)" title="(.*?)".*?data-original="(.*?)".*?class="dur">(.*?)</span>.*?class="views">((?:\d+\s|)\d+) views</div>', data, re.S)
		if Movies:
			for (Url, Title, Image, Runtime, Views) in Movies:
				self.filmliste.append((decodeHtml(Title), Url, Image, Runtime, Views.replace(' ','')))
		if len(self.filmliste) == 0:
			self.filmliste.append((_('No videos found!'), None, None, '', ''))
		self.ml.setList(map(self._defaultlistleft, self.filmliste))
		self.ml.moveToIndex(0)
		self.keyLocked = False
		self.th_ThumbsQuery(self.filmliste, 0, 1, 2, None, None, self.page, self.lastpage, mode=1)
		self.showInfos()

	def showInfos(self):
		title = self['liste'].getCurrent()[0][0]
		url = self['liste'].getCurrent()[0][1]
		pic = self['liste'].getCurrent()[0][2]
		runtime = self['liste'].getCurrent()[0][3]
		views = self['liste'].getCurrent()[0][4]
		self['name'].setText(title)
		self['handlung'].setText("Runtime: %s\nViews: %s" % (runtime, views))
		CoverHelper(self['coverArt']).getCover(pic)

	def keyOK(self):
		if self.keyLocked:
			return
		Link = self['liste'].getCurrent()[0][1]
		if Link:
			self['name'].setText(_('Please wait...'))
			twAgentGetPage(Link, agent=agent, cookieJar=cookies).addCallback(self.parseVideo).addErrback(self.dataError)

	def parseVideo(self, data, referer=''):
		if "Embed Player Error" in data:
			message = self.session.open(MessageBoxExt, _("Stream not found"), MessageBoxExt.TYPE_INFO, timeout=5)
		else:
			license = re.findall('license_code:\s\'(.*?)\',', data, re.S)
			url = re.findall('video_(?:alt_|)url\d{0,1}:\s\'(.*?)\'.*?video_(?:alt_|)url_text:\s\'(\d+)(?:p|)\',', data, re.S)
			if not url:
				url = re.findall('video_(?:alt_|)url\d{0,1}:\s\'(.*?)\'', data, re.S)
			if license and url:
				try:
					max = 0
					for vid in url:
						if int(vid[1]) > max:
							max = int(vid[1])
							url = vid[0]
				except:
					url = url[-1]
				if 'function/0/' in url:
					url = decrypturl(url, license[0])
				if url:
					tw_agent_hlp = TwAgentHelper(cookieJar=cookies)
					tw_agent_hlp.getRedirectedUrl(url).addCallback(self.getStream, referer).addErrback(self.dataError)
			else:
				fetch = re.findall('<iframe.*?src="(http://(?:webcamsdolls.com|www.camhub.world|www.zzpornozz.xyz)/embed/\d+)', data, re.S)
				if fetch:
					url = fetch[0]
					twAgentGetPage(url, agent=agent).addCallback(self.parseVideo, url).addErrback(self.dataError)
				else:
					message = self.session.open(MessageBoxExt, _("Stream not found"), MessageBoxExt.TYPE_INFO, timeout=5)

	def getStream(self, url, referer):
		self['name'].setText('')
		Title = self['liste'].getCurrent()[0][0]
		Link = self['liste'].getCurrent()[0][1]
		import requests
		ck = requests.utils.dict_from_cookiejar(cookies)
		headers = '&Cookie=%s' % ','.join(['%s=%s' % (key, urllib.quote_plus(ck[key])) for key in ck])
		if referer == '':
			referer = Link
		headers = headers + '&Referer=%s' % referer
		url = url + '#User-Agent='+agent+headers
		mp_globals.player_agent = agent
		self.session.open(SimplePlayer, [(Title, url)], showPlaylist=False, ltype='camhub')